/*
Copyright © 2020-2023 Hannes Hayashi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package cmd

import (
	"log"
	"sync"

	"github.com/hanneshayashi/gsm/gsmdrive"
	"github.com/hanneshayashi/gsm/gsmhelpers"

	"github.com/spf13/cobra"
	"google.golang.org/api/drive/v3"
)

// filesRemoveLabelsBatchCmd represents the batch command
var filesRemoveLabelsBatchCmd = &cobra.Command{
	Use:   "batch",
	Short: "Batch remove the specified labels on files using a CSV file as input.",
	Long:  "Implements the API documented at https://developers.google.com/drive/api/v3/reference/files/modifyLabels",
	Annotations: map[string]string{
		"crescendoAttachToParent": "true",
	},
	DisableAutoGenTag: true,
	Run: func(cmd *cobra.Command, _ []string) {
		maps, err := gsmhelpers.GetBatchMaps(cmd, fileFlags)
		if err != nil {
			log.Fatalln(err)
		}
		var wg sync.WaitGroup
		cap := cap(maps)
		type resultStruct struct {
			FileID string         `json:"fileId,omitempty"`
			Labels []*drive.Label `json:"labels,omitempty"`
		}
		results := make(chan resultStruct, cap)
		go func() {
			for i := 0; i < cap; i++ {
				wg.Add(1)
				go func() {
					for m := range maps {
						fileID := m["fileId"].GetString()
						req, err := mapToRemoveLabelsRequest(m)
						if err != nil {
							log.Printf("Error building remove labels request: %v", err)
							continue
						}
						r := resultStruct{FileID: fileID}
						result, err := gsmdrive.ModifyLabels(fileID, m["fields"].GetString(), req)
						if err != nil {
							log.Println(err)
						} else {
							r.Labels = result
						}
						results <- r
					}
					wg.Done()
				}()
			}
			wg.Wait()
			close(results)
		}()
		if streamOutput {
			enc := gsmhelpers.GetJSONEncoder(false)
			for r := range results {
				err := enc.Encode(r)
				if err != nil {
					log.Println(err)
				}
			}
		} else {
			final := []resultStruct{}
			for res := range results {
				final = append(final, res)
			}
			err := gsmhelpers.Output(final, "json", compressOutput)
			if err != nil {
				log.Fatalln(err)
			}
		}
	},
}

func init() {
	gsmhelpers.InitBatchCommand(filesRemoveLabelsCmd, filesRemoveLabelsBatchCmd, fileFlags, fileFlagsALL, batchFlags)
}
